package cat.hack3.codingtests.marsrover;

import cat.hack3.codingtests.marsrover.api.RotableRover;
import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import cat.hack3.codingtests.marsrover.api.commands.RoverActionReport;
import cat.hack3.codingtests.marsrover.api.commands.RoverActionReport.ReportType;

public class RoverActionReporter {
    private static final String OBSTACLE_REPORT_MESSAGE = "There is an obstacle at %s, therefore I will keep the same position";
    private static final String MOVEMENT_REPORT_MESSAGE = "Updated coordinates towards %s, now is %s-%s";
    public static final String ROTATION_REPORT_MESSGE = "Rotated towards %s, now the direction is %s";

    public RoverActionReport reportObstacle(Coordinates newPosition) {
        var collisionReport = String.format(OBSTACLE_REPORT_MESSAGE, newPosition);
        return reportWarning(collisionReport);
    }

    public RoverActionReport createMovementReport(Direction direction, Coordinates newPosition) {
        String movementReport = String.format(MOVEMENT_REPORT_MESSAGE,
                direction, newPosition.latitude(), newPosition.longitude());
        return reportInfo(movementReport);
    }

    public RoverActionReport createRotationReport(RotableRover.Rotation rotation, Direction newDirection) {
        return reportInfo(String.format(ROTATION_REPORT_MESSGE,
                rotation, newDirection));
    }

    private RoverActionReport reportInfo(String message) {
        return new RoverActionReport(ReportType.INFO, message);
    }

    private RoverActionReport reportWarning(String message) {
        return new RoverActionReport(ReportType.WARNING, message);
    }
}
