package cat.hack3.codingtests.marsrover.gps;

import cat.hack3.codingtests.marsrover.api.cartography.Coordinates.Latitude;
import cat.hack3.codingtests.marsrover.api.cartography.Coordinates.Longitude;

public class MapIncrementalPositionResolver {

    private final int firstPositionInMap;
    private final int incrementUnit;
    private final long maximumYPosition;
    private final long maximumXPosition;

    public MapIncrementalPositionResolver(int firstPositionInMap, int incrementUnit, long maximumYPosition, long maximumXPosition) {
        this.firstPositionInMap = firstPositionInMap;
        this.incrementUnit = incrementUnit;
        this.maximumYPosition = maximumYPosition;
        this.maximumXPosition = maximumXPosition;
    }

    Latitude getIncremented(Latitude latitude) {
        long incrementedPosition = getIncremented(latitude.pointer(), maximumYPosition);
        return Latitude.of(incrementedPosition);
    }

    Longitude getIncremented(Longitude longitude) {
        long incrementedPosition = getIncremented(longitude.pointer(), maximumXPosition);
        return Longitude.of(incrementedPosition);
    }

    private long getIncremented(long currentPosition, long threshold) {
        return currentPosition + incrementUnit > threshold
                ? firstPositionInMap
                : ++currentPosition;
    }

    Latitude getDecremented(Latitude latitude) {
        long decrementedPosition = getDecremented(latitude.pointer(), maximumYPosition);
        return Latitude.of(decrementedPosition);
    }

    Longitude getDecremented(Longitude longitude) {
        long decrementedPosition =  getDecremented(longitude.pointer(), maximumXPosition);
        return Longitude.of(decrementedPosition);
    }

    private long getDecremented(long currentPosition, long threshold) {
        return currentPosition - incrementUnit < firstPositionInMap
                ? threshold
                : --currentPosition;
    }
}
