package cat.hack3.codingtests.marsrover.test;

import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import cat.hack3.codingtests.marsrover.test.context.GivenRotableAndMoveCommandsOnDeployedRoverWithMap;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static cat.hack3.codingtests.marsrover.api.cartography.Direction.*;
import static org.testng.Assert.assertEquals;

public class MarsRoverTest extends GivenRotableAndMoveCommandsOnDeployedRoverWithMap {

    @BeforeMethod
    public void setUp() {
        super.setUp();
    }

    @Test
    public void stepForward() {
        Direction originalDirection = rover.getCurrentDirection();

        executeCommandAndReport(moveForwardCommand);

        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 3));
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void stepBackwards() {
        Direction originalDirection = rover.getCurrentDirection();

        executeCommandAndReport(moveBackwardsCommand);
        Coordinates currentPosition = rover.getCurrentCoordinates();

        assertEquals(currentPosition, Coordinates.of(1, 3));
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void turnLef() {
        executeCommandAndReport(turnLeftCommand);
        assertEquals(rover.getCurrentDirection(), EAST);
        executeCommandAndReport(turnLeftCommand);
        assertEquals(rover.getCurrentDirection(), NORTH);
        executeCommandAndReport(turnLeftCommand);
        assertEquals(rover.getCurrentDirection(), WEST);
        executeCommandAndReport(turnLeftCommand);
        assertEquals(rover.getCurrentDirection(), SOUTH);
        executeCommandAndReport(turnLeftCommand);
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void turnRight() {
        executeCommandAndReport(turnRightCommand);
        assertEquals(rover.getCurrentDirection(), WEST);
        executeCommandAndReport(turnRightCommand);
        assertEquals(rover.getCurrentDirection(), NORTH);
        executeCommandAndReport(turnRightCommand);
        assertEquals(rover.getCurrentDirection(), EAST);
        executeCommandAndReport(turnRightCommand);
        assertEquals(rover.getCurrentDirection(), SOUTH);
        executeCommandAndReport(turnRightCommand);
        assertEquals(rover.getCurrentDirection(), WEST);
    }

    @Test
    public void loopTheWorldInLatitudeMovingForward() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        repeatAction(10, () -> executeCommandAndReport(moveForwardCommand));

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void loopTheWorldInLatitudeMovingBackwards() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        repeatAction(10, () -> executeCommandAndReport(moveBackwardsCommand));

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void loopTheWorldInLongitudeMovingForward() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();

        executeCommandAndReport(turnLeftCommand);
        repeatAction(10, () -> executeCommandAndReport(moveForwardCommand));

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void loopTheWorldInLongitudeMovingBackwards() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();

        executeCommandAndReport(turnLeftCommand);
        repeatAction(10, () -> executeCommandAndReport(moveBackwardsCommand));

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void acceptanceTest() {
        executeCommandAndReport(moveForwardCommand);
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 3));
        assertEquals(rover.getCurrentDirection(), SOUTH);

        executeCommandAndReport(turnLeftCommand);
        repeatAction(23, () -> executeCommandAndReport(moveForwardCommand));
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 6));
        assertEquals(rover.getCurrentDirection(), EAST);

        repeatAction(3, () -> executeCommandAndReport(turnRightCommand));
        repeatAction(9, () -> executeCommandAndReport(moveForwardCommand));
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(4, 6));
        assertEquals(rover.getCurrentDirection(), NORTH);

        executeCommandAndReport(moveBackwardsCommand);
        executeCommandAndReport(turnRightCommand);
        executeCommandAndReport(moveForwardCommand);
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(5, 7));
        assertEquals(rover.getCurrentDirection(), EAST);
    }

}
