package cat.hack3.codingtests.marsrover.test;

import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import cat.hack3.codingtests.marsrover.test.context.GivenRotableAndMoveCommandsOnDeployedRoverWithMap;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotEquals;

public class MarsRoverWithObstaclesTest extends GivenRotableAndMoveCommandsOnDeployedRoverWithMap {

    @Override
    protected List<Coordinates> getObstacles() {
        return List.of(
                Coordinates.of(3, 3),
                Coordinates.of(5, 5)
        );
    }

    @BeforeMethod
    public void setUp() {
        super.setUp();
    }

    @Test
    public void findAnObstacleAtFirstMove() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        executeCommandAndReport(moveForwardCommand);

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void afterFindingObstacleWalkAroundUntilFindingNext() {
        executeCommandAndReport(moveForwardCommand);
        Coordinates oneStepForward = rover.getCurrentCoordinates();
        assertNotEquals(oneStepForward, Coordinates.of(3, 3));
        assertEquals(oneStepForward, Coordinates.of(2, 3));

        executeCommandAndReport(turnLeftCommand);
        repeatAction(2, () -> executeCommandAndReport(moveForwardCommand));
        executeCommandAndReport(turnRightCommand);
        repeatAction(2, () -> executeCommandAndReport(moveForwardCommand));
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(4, 5));
        executeCommandAndReport(moveForwardCommand);
        assertNotEquals(rover.getCurrentCoordinates(), Coordinates.of(5, 5));
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(4, 5));
    }
}
